import { inspection } from "../../../../entity/unit/inspection/inspection";
import { InspectionRelated, InspectionViewModel } from "../../../../viewmodel/admin/unit/inspection/inspection.models";
import EquipmentFactory from "../equipment/equipment";
import VehicleFactory from "../vehicle/vehicle";
import WearableFactory from "../wearable/wearable";
import InspectionPlanFactory from "./inspectionPlan";
import InspectionPointResultFactory from "./inspectionPointResult";
import InspectionVersionedPlanFactory from "./inspectionVersionedPlan";

export default abstract class InspectionFactory {
  /**
   * @description map record to inspection
   * @param {inspection} record
   * @returns {InspectionViewModel}
   */
  public static mapToSingle(record: inspection): InspectionViewModel {
    let related: InspectionRelated;
    if (record?.equipmentId) {
      related = {
        relatedId: record.equipmentId,
        assigned: "equipment",
        related: EquipmentFactory.mapToSingle(record.equipment),
      };
    } else if (record?.vehicleId) {
      related = {
        relatedId: record.vehicleId,
        assigned: "vehicle",
        related: VehicleFactory.mapToSingle(record.vehicle),
      };
    } else {
      related = {
        relatedId: record.wearableId,
        assigned: "wearable",
        related: WearableFactory.mapToSingle(record.wearable),
      };
    }

    return {
      id: record.id,
      inspectionPlanId: record.inspectionPlanId,
      inspectionPlan: InspectionPlanFactory.mapToSingle(record.inspectionPlan),
      inspectionVersionedPlanId: record.inspectionVersionedPlanId,
      inspectionVersionedPlan: InspectionVersionedPlanFactory.mapToSingle(record.inspectionVersionedPlan),
      context: record.context,
      created: record.createdAt,
      finished: record?.finishedAt,
      isOpen: record?.finishedAt == undefined,
      nextInspection: record?.nextInspection,
      checks: InspectionPointResultFactory.mapToBase(record.pointResults),
      ...related,
    };
  }

  /**
   * @description map records to inspection
   * @param {Array<inspection>} records
   * @returns {Array<InspectionViewModel>}
   */
  public static mapToBase(records: Array<inspection>): Array<InspectionViewModel> {
    return records.map((r) => this.mapToSingle(r));
  }
}
